#!/usr/bin/perl -w

use strict;

#caad.arch.ethz.ch // Steffen Lemmerzahl // 2005.12.09
print "\n[KaisersRot.CAAD.ETHZ] Bitmap2XML starting up...\n\n";

use XML::Simple;
use Imager qw(:handy);

################################################################################
# PROGRAMM STARTS HERE!
################################################################################

# open the settingsfile
################################################################################

my $file = shift;
open(my $prefs, "<", $file) or die "\n\nSettings not found, please use syntax: perl bitmap2xml.pl <settings>\n\n",$!;

my @settings = <$prefs>;

$file = $settings[1];	#name of output xml
my $hoehe = $settings[3]+0;	#height of output in meters
my $breite = $settings[5]+0;	#width of output in meters
my $ifile = $settings[7];	#name if input file (must be .BMP)
my $rax = $settings[9]+0;	#Raster X
my $ray = $settings[11]+0;	#Raster Y
close($prefs);

$ifile =~ s/^\s*//;
$ifile =~ s/\s*$//;
$file =~ s/^\s*//;
$file =~ s/\s*$//;

open(OUTPUT, ">$file") || die "\ncan't create the file $file!\n";
print "\na file named: $file has been generated\n\n";

# read in image and rasterize
################################################################################

my $image_in = Imager->new();

$image_in->open(file=>$ifile) or die $image_in->errstr();

my $width = $image_in->getwidth();
my $height = $image_in->getheight();

my $xl=$width;
my $yo=$height;
my $xr=0;
my $yu=0;
my $ii;
my $i;

my @color = ();
my $gray;
my @graymap = ();

print "\n...reading in image $ifile, this may take a while!\n";

for ($i=0; $i<$height; $i++)
{
	for ($ii=1; $ii<$width; $ii++)
	{

		@color = $image_in->getpixel(x=>$ii, y=>$i)->rgba();
		$graymap[$i][$ii]  = ($color[0] + $color[1] + $color[2]) / (3 * 255);

		if ($graymap[$i][$ii] < 1) {
			if ($ii<$xl) {$xl=$ii;}
			if ($ii>$xr) {$xr=$ii;}
			if ($i<$yo) {$yo=$i;}
			if ($i>$yu) {$yu=$i;}
			}

	}
}
print "\nImage Width ",$width,"\nImage Height ",$height,"\nImage Size ",$width*$height,"\n";

print "\nBounding Box :(",$xl,",",$yo,"),(",$xr,",",$yu,")\n";

# adapt raster to content size
################################################################################

my $blockx = ($xr-$xl)/$rax;
my $blocky = ($yu-$yo)/$ray;

print "\nBlock X: $blockx / Block Y: $blocky\n";

# create arry containing interpolated grid
################################################################################

my $j;
my $jj;
my @raster = ();
my $summe;
my $tempy;
my $tempx;

for ($i=0; $i<$ray; $i++)
{
	$tempy = int($blocky * $i);
	for ($ii=0; $ii<$rax; $ii++)
	{
		$tempx = int($blockx * $ii);
		$summe = 0;
		#@summe=(0,0,0);
		
		for ($j=($xl+$tempx); $j<($xl+$tempx+$blockx); $j++)
		{
			for ($jj=($yo+$tempy); $jj<($yo+$tempy+$blocky);$jj++)
			{
				$summe+=$graymap[$jj][$j];
			}
		}
		$raster[$i][$ii]=$summe/((int($blockx)+1)*(int($blocky)+1));
		#print "\nRaster X:$ii Y:$i von ",$xl+$tempx,",",$yo+$tempy," bis ",$xl+$tempx+$blockx,",",$yo+$tempy+$blocky," mit wert: ",$raster[$i][$ii];		
	}
}	

# write out XML
################################################################################

my $xmldata;
my $id = 0;

for ($i=0; $i<$ray; $i++)
{
	for ($ii=0; $ii<$rax; $ii++)
	{
		$xmldata->{CELL}[$id]->{ID}[0]=$id;
		$xmldata->{CELL}[$id]->{X}[0]=$ii;
		$xmldata->{CELL}[$id]->{Y}[0]=$i;
		$xmldata->{CELL}[$id]->{VALUE}[0]=$raster[$i][$ii];
		$id++;				
	}
}
print OUTPUT XMLout($xmldata, rootname =>"RASTER");

close(OUTPUT);